searchNodes=[{"doc":"MockeryExtras.Given provides a simple way to stub function calls.use Given given Map.get(%{}, :key), return: &quot;5&quot; given Map.get(@any, :key), return: &quot;5&quot;There is also support for writing your own stubbing macros.MockeryExtras.Getters provides shorthand for defining getters for nodes in complex structures. With a small amount of copying and pasting, you can isolate both client code and tests from details about structure. See Stubbing Complex Structures for an example.defmodule EctoTestDSL.Run.RunningExample do defstruct [:example, :history, script: :none_just_testing, tracer: :none] getters :example, [ # &lt;&lt;&lt;&lt;&lt;&lt;&lt;&lt;&lt;&lt; eens: [], validation_changeset_checks: [], constraint_changeset_checks: [], field_checks: %{}, fields_from: :nothing, ]","ref":"MockeryExtras.html","title":"MockeryExtras","type":"module"},{"doc":"Utilities for defining functions that get map/struct/keyword values up to three levels of nesting. Even though complex nested structures aren't necessarily the best thing for your code, they sometimes exist: all = %{ top1: 1, top2: 1.2, down: %{ lower: 2, down: %{ lowest: 3 } } }You're stuck with them, at least for now. You'd often prefer not to spread knowledge of the exact structure through the code, because that would make a lot of code fragile in the face of change. So you can hide the structure behind &quot;getters&quot; that (alone) know how to reach into the structure and retrieve a value. They present a &quot;flattened&quot; API to client code.Those functions are simple to write manually, but you might as well use a macro to do it for you:getters([:top1, :top2]) # defines top1/1, top2/1, so that: # top1(all) =&gt; 1 # top2(all) =&gt; 2 # Three levels of nesting are allowed. getters(:down, :down, [:lowest]) # lowest(all) =&gt; 3See getters/1 for details, such as how to specify default values (akin to those of Map.get/3 or Keyword.get/3).See Stubbing Complex Structures to see how a little bit of custom (copy and tweak) work can improve the testing of code that works against a flattened API, plus an alternative to verbose code like MyStructure.getter(structure).","ref":"MockeryExtras.Getters.html","title":"MockeryExtras.Getters","type":"module"},{"doc":"Define a single getter with a different name than the field it accesses.For example, this:getter :meio, for: [:root, :middle] # meio(root: %{middle: 3}) =&gt; 3A default can also be provided:getter :meio, for: [:root, :middle], default: &quot;default&quot; # meio(root: %{}) =&gt; &quot;default&quot;If the getter is for the top level, the for: value needn't be a list:getter :raiz, for: :root","ref":"MockeryExtras.Getters.html#getter/2","title":"MockeryExtras.Getters.getter/2","type":"macro"},{"doc":"Define N getters, possibly with defaults.The argument must be a list. For each atom in the list, a function effectively like the following is defined:def atom(structure), do: structure[atom]Note that the getter's argument can be either a Map or a Keyword. Default values can be given:getters([:x, y: &quot;some default&quot;]) # y(%{}) =&gt; &quot;some default&quot;If there's no default, a missing value will produce a KeyError.","ref":"MockeryExtras.Getters.html#getters/1","title":"MockeryExtras.Getters.getters/1","type":"macro"},{"doc":"Define N getters for the second level of a nested structure.The first argument is an atom that's expected to index a Map or Keyword. The value of that index is again expected to be a Map or Keyword.The second argument is a list with the same form as in getters/1. It provides the names for the generated functions. (The first argument plays no role.)getters(:top, [:x, y: &quot;some default&quot;]) # x(%{top: [x: 1]}) =&gt; 1 # y(%{top: [ ]}) =&gt; &quot;some default&quot;Note that the default only applies at the bottom level. The following will raise a RuntimeError:","ref":"MockeryExtras.Getters.html#getters/2","title":"MockeryExtras.Getters.getters/2","type":"macro"},{"doc":"Define N getters for the third level of a nested structure.","ref":"MockeryExtras.Getters.html#getters/3","title":"MockeryExtras.Getters.getters/3","type":"macro"},{"doc":"The same as getters/1 except the functions are generated using defp.Most often this is used in a module that defines a structure, its getters, and also more complicated functions that work on the structure.","ref":"MockeryExtras.Getters.html#private_getters/1","title":"MockeryExtras.Getters.private_getters/1","type":"macro"},{"doc":"The same as getters/2 except the functions are generated using defp.","ref":"MockeryExtras.Getters.html#private_getters/2","title":"MockeryExtras.Getters.private_getters/2","type":"macro"},{"doc":"The same as getters/3 except the functions are generated using defp.","ref":"MockeryExtras.Getters.html#private_getters/3","title":"MockeryExtras.Getters.private_getters/3","type":"macro"},{"doc":"This module provides a pretty notation for a common case that's not gracefully handled by Mockery. Here is how you instruct Mockery to return the value &quot;5&quot; when Map.get/3 is called with %{} and :key:use MockeryExtras.Given given Map.get(%{}, :key), return: &quot;5&quot;Note that the first argument to given looks like an ordinary function call.It is also common to have a &quot;don't care&quot; argument, like this:given Map.get(@any, :key), return: &quot;5&quot;See given/2 for more.","ref":"MockeryExtras.Given.html","title":"MockeryExtras.Given","type":"module"},{"doc":"The guts of given/2 for use in your own macros.This function is convenient when you want to create a number of stubs at once. For example, suppose the RunningExample module has several single-argument getters. A stub macro can be more compact than several givens:stub( original_params: input, format: :phoenix, neighborhood: %{een(b: Module) =&gt; %{id: 383}})stub can be written like this:defmacro stub(kws) do for {key, val} &lt;- kws do Given.expand(RunningExample, [{key, 1}], [:running], val) end endWhen calling expand(module_alias, name_and_arity, arglist_spec, return_value), know that:module_alias can be a simple atom, like RunningExample, which is an alias for EctoTestDSL.Run.RunningExample. More generally, it can be the :__aliases__ value from Macro.decompose_call/1. name_and_arity is a function name and arity pair of the form [get: 3].arglist_spec is a list of values like [5, @any].return_value is any value.","ref":"MockeryExtras.Given.html#expand/4","title":"MockeryExtras.Given.expand/4","type":"function"},{"doc":"Takes what looks like a function call, plus a return value, and arranges that such a function call will return the given value whenever it's made at a &quot;seam&quot; marked with Mockery.mockable. # Code: ... mockable(Schema).changeset(struct, params) ... # Test: given Schema.changeset(%Schema{}, %{age: &quot;3&quot;}), return: %Changeset{...}The function's arguments and return value can be constants, as shown above, or they can be calculations or variables. That can be helpful when the given appears in a test helper:def helper(params, cast_value, something_else) do ... given Schema.changeset(%Schema{}, params), return: cast_value ... assert ... endA function argument can be the special value @any (defined when the module is used). That's useful when the argument is irrelevant and you don't want to have to type it out: given Schema.changeset(@any, params), return: cast_value@any expands to a function whose value is always true. More generally, any function used as an argument is not matched with equality. Instead, the call-time value is passed to the function, which should return a truthy value to indicate a match. So you can do this: given Module.f(5, &amp;even/1), return: 8Notes:You can provide return values for many arglist values. given Module.f(5, &amp;even/1), return: 8 given Module.f(6, @any), return: 9If there's more than one match, the first is used.If the same arglist is given twice, the second replaces the first. That lets you use ExUnit setup to establish defaults:def setup do given RunningExample.params(:a_runnable), return: %{} ... test &quot;...&quot; given RunningExample.params(:a_runnable), return: %{&quot;a&quot; =&gt; &quot;1&quot;} assert Steps.runnable(:a_runnable) == %{a: 1} endIf a function has a given value for one or more arglists, but none matched, an error is raised.","ref":"MockeryExtras.Given.html#given/2","title":"MockeryExtras.Given.given/2","type":"macro"},{"doc":"This shows (as with IO.inspect) all the existing stubs.The format is not pretty.[ {{Given, Date, [add: 2]}, [ {[~D[2001-02-03], 3], &quot;return for 3&quot;, #Function&lt;9.8563522/1 in MockeryExtras.Stubbery.make_matcher/1&gt;} ]} ]","ref":"MockeryExtras.Given.html#inspect/0","title":"MockeryExtras.Given.inspect/0","type":"function"}]